#include <SPI.h>
#include <SD.h>
#include <Wire.h>
#include "RTClib.h"

// A simple data logger for the Arduino analog pins

// how many milliseconds between grabbing data and logging it. 1000 ms is once a second
#define LOG_INTERVAL  1000 // mills between entries (reduce to take more/faster data)

// how many milliseconds before writing the logged data permanently to disk
// set it to the LOG_INTERVAL to write each time (safest)
// set it to 10*LOG_INTERVAL to write all data every 10 datareads, you could lose up to 
// the last 10 reads if power is lost but it uses less power and is much faster!
#define SYNC_INTERVAL 1000 // mills between calls to flush() - to write data to the card
uint32_t syncTime = 0; // time of last sync()

#define ECHO_TO_SERIAL   1 // echo data to serial port

// the digital pins that connect to the LEDs
#define redLEDpin 2
#define greenLEDpin 3

// The analog pins that connect to the sensors
#define mantaChargerCurrentPin 0           // analog 0
#define mantaChargerVoltagePin 1

// how many samples per average value
#define nSamples 15

// Current probe ranges
float range5A = 0.185;
float range20A = 0.1;
float range30A = 0.066;

float voltageScaleFactor = 12*0.00489;  //5/1023;
//float voltageScaleFactor = 0.00489;  //5/1023;

RTC_DS1307 RTC; // define the Real Time Clock object

// for the data logging shield, we use digital pin 10 for the SD cs line
const int chipSelect = 10;

// the logging file
File logfile;

void error(char *str)
{
  Serial.print("error: ");
  Serial.println(str);
  
  // red LED indicates error
  digitalWrite(redLEDpin, HIGH);

  while(1);
}


void setup(void)
{
  Serial.begin(9600);
  Serial.println();
  
  // use debugging LEDs
  pinMode(redLEDpin, OUTPUT);
  pinMode(greenLEDpin, OUTPUT);
  
  // initialize the SD card
  Serial.print("Initializing SD card...");
  // make sure that the default chip select pin is set to
  // output, even if you don't use it:
  pinMode(10, OUTPUT);
  
  // see if the card is present and can be initialized:
  if (!SD.begin(chipSelect)) {
    error("Card failed, or not present");
  }
  Serial.println("card initialized.");
  
  // create a new file
  char filename[] = "LOGGER00.CSV";
  for (uint8_t i = 0; i < 100; i++) {
    filename[6] = i/10 + '0';
    filename[7] = i%10 + '0';
    if (! SD.exists(filename)) {
      // only open a new file if it doesn't exist
      logfile = SD.open(filename, FILE_WRITE); 
      break;  // leave the loop!
    }
  }
  
  if (! logfile) {
    error("couldnt create file");
  }
  
  Serial.print("Logging to: ");
  Serial.println(filename);

  // connect to RTC
  Wire.begin();  
  if (!RTC.begin()) {
    logfile.println("RTC failed");
#if ECHO_TO_SERIAL
    Serial.println("RTC failed");
#endif  //ECHO_TO_SERIAL
  }
  

  logfile.println("millis,stamp,datetime,mantaChargerCurrent,mantaChargerVoltage");    
#if ECHO_TO_SERIAL
  Serial.println("millis,stamp,datetime,mantaChargerCurrent,mantaChargerVoltage");  
#endif //ECHO_TO_SERIAL
 
  // If you want to set the aref to something other than 5v
  // analogReference(EXTERNAL);
}

void loop(void)
{
  DateTime now;

  // delay for the amount of time we want between readings
  delay((LOG_INTERVAL -1) - (millis() % LOG_INTERVAL));
  
  digitalWrite(greenLEDpin, HIGH);
  
  // log milliseconds since starting
  uint32_t m = millis();
  logfile.print(m);           // milliseconds since start
  logfile.print(", ");    
#if ECHO_TO_SERIAL
  Serial.print(m);         // milliseconds since start
  Serial.print(", ");  
#endif

  // fetch the time
  now = RTC.now();
  // log time
  logfile.print(now.unixtime()); // seconds since 1/1/1970
  logfile.print(", ");
  logfile.print('"');
  logfile.print(now.year(), DEC);
  logfile.print("/");
  logfile.print(now.month(), DEC);
  logfile.print("/");
  logfile.print(now.day(), DEC);
  logfile.print(" ");
  logfile.print(now.hour(), DEC);
  logfile.print(":");
  logfile.print(now.minute(), DEC);
  logfile.print(":");
  logfile.print(now.second(), DEC);
  logfile.print('"');
#if ECHO_TO_SERIAL
  Serial.print(now.unixtime()); // seconds since 1/1/1970
  Serial.print(", ");
  Serial.print('"');
  Serial.print(now.year(), DEC);
  Serial.print("/");
  Serial.print(now.month(), DEC);
  Serial.print("/");
  Serial.print(now.day(), DEC);
  Serial.print(" ");
  Serial.print(now.hour(), DEC);
  Serial.print(":");
  Serial.print(now.minute(), DEC);
  Serial.print(":");
  Serial.print(now.second(), DEC);
  Serial.print('"');
#endif //ECHO_TO_SERIAL

// Manta Charger Current Reading
unsigned int x=0;
float mantaChargerCurrentValue=0.0,mantaChargerCurrentSamples=0.0,mantaChargerCurrentValueAvg=0.0,mantaChargerCurrentOutput=0.0;

  for (int x = 0; x < nSamples; x++){ //Get 150 samples
  mantaChargerCurrentValue = analogRead(mantaChargerCurrentPin);     //Read current sensor values   
  mantaChargerCurrentSamples = mantaChargerCurrentSamples + mantaChargerCurrentValue;  //Add samples together
  delay (3); // let ADC settle before next sample 3ms
}
mantaChargerCurrentValueAvg=mantaChargerCurrentSamples/nSamples;//Taking Average of Samples

//((AvgAcs * (5.0 / 1024.0)) is converitng the read voltage in 0-5 volts
//2.5 is offset(I assumed that arduino is working on 5v so the viout at no current comes
//out to be 2.5 which is out offset. If your arduino is working on different voltage than 
//you must change the offset according to the input voltage)
//0.066v(66mV) is rise in output voltage when 1A current flows at input

mantaChargerCurrentOutput = -(2.5 - (mantaChargerCurrentValueAvg * (5.0 / 1024.0)) )/range20A;

// Manta Charger Voltage Reading
unsigned int y=0;
float mantaChargerVoltageValue=0.0,mantaChargerVoltageSamples=0.0,mantaChargerVoltageValueAvg=0.0,mantaChargerVoltageOutput=0.0;

  for (int x = 0; x < nSamples; x++){ //Get 150 samples
  mantaChargerVoltageValue = analogRead(mantaChargerVoltagePin);     //Read current sensor values   
  mantaChargerVoltageSamples = mantaChargerVoltageSamples + mantaChargerVoltageValue;  //Add samples together
  delay (3); // let ADC settle before next sample 3ms
}
mantaChargerVoltageValueAvg=mantaChargerVoltageSamples/nSamples;//Taking Average of Samples

//((AvgAcs * (5.0 / 1024.0)) is converitng the read voltage in 0-5 volts
//2.5 is offset(I assumed that arduino is working on 5v so the viout at no current comes
//out to be 2.5 which is out offset. If your arduino is working on different voltage than 
//you must change the offset according to the input voltage)
//0.066v(66mV) is rise in output voltage when 1A current flows at input



mantaChargerVoltageOutput = mantaChargerVoltageValueAvg*voltageScaleFactor;

// One reading 

//  analogRead(mantaChargerCurrentPin);
//  delay(10); 
//  int sensorReading1 = analogRead(mantaChargerCurrentPin);  


  logfile.print(", ");    
  logfile.print(mantaChargerCurrentOutput);
//  logfile.print(", ");    
//  logfile.print(temperatureF);
#if ECHO_TO_SERIAL
  Serial.print(", ");   
  Serial.print(mantaChargerCurrentOutput);
//  Serial.print(", ");    
//  Serial.print(temperatureF);
#endif //ECHO_TO_SERIAL

  logfile.print(", ");    
  logfile.print(mantaChargerVoltageOutput);
//  logfile.print(", ");    
//  logfile.print(temperatureF);
#if ECHO_TO_SERIAL
  Serial.print(", ");   
  Serial.print(mantaChargerVoltageOutput);
//  Serial.print(", ");    
//  Serial.print(temperatureF);
#endif //ECHO_TO_SERIAL

  logfile.println();
#if ECHO_TO_SERIAL
  Serial.println();
#endif // ECHO_TO_SERIAL

  digitalWrite(greenLEDpin, LOW);

  // Now we write data to disk! Don't sync too often - requires 2048 bytes of I/O to SD card
  // which uses a bunch of power and takes time
  if ((millis() - syncTime) < SYNC_INTERVAL) return;
  syncTime = millis();
  
  // blink LED to show we are syncing data to the card & updating FAT!
  digitalWrite(redLEDpin, HIGH);
  logfile.flush();
  digitalWrite(redLEDpin, LOW);
  
}
